==> live-elements-js-compiler-0.1.9.tgz: package/lib/lvelements/compiler/src/cursorcontext.cpp <==
/****************************************************************************
**
** Copyright (C) 2022 Dinu SV.
** This file is part of Livekeys Application.
**
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPLv3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl.html.
**
****************************************************************************/

#include "cursorcontext.h"

/**
 * \class lv::el::CursorContext
 * \brief Provides completion context info
 *
 * Cursor context provides information about a given cursor position within an elements document.
 * CursorContext is obtained from a parsed document, which extracts all the available information
 * from the AST.
 *
 * The class provides additional information depending on the context:
 *
 *
 *
 * Within an import:
 *
 * \code
 * import path.to.import
 *                  ^ context = CompletionContext::InImport | CompletionContext::InElements
 *                    expressionPath = {RangeFor<path>, RangeFor<to>, RangeFor<im>}
 * \endcode
 *
 * \code
 * import path.to.import
 *                ^ context = CompletionContext::InImport | CompletionContext::InElements
 *                  expressionPath = {RangeFor<path>, RangeFor<to>, Range()}
 * \endcode
 *
 * \code
 * import path.to.import
 *        ^ context = CompletionContext::InImport | CompletionContext::InElements
 *          expressionPath = {}
 * \endcode
 *
 * \code
 * import path.to.import
 *     ^ context = 0
 * \endcode
 *
 *
 *
 *
 * Within a string
 *
 * \code
 * var a = "example"
 *           ^ context = CompletionContext::InString
 * \endcode
 *
 *
 *
 *
 * Left of a declaration
 *
 * \code
 * component T < Element{ prop : 200 }
 *                          ^ context = CompletionContext::InLeftOfDeclaration | CompletionContext::InElements
 *                            expressionPath = {RangeFor<pr>}
 *                            objectType = RangeFor<Element>
 * \endcode
 *
 * \code
 * component T < Element{ int prop : 200 }
 *                              ^ context = CompletionContext::InLeftOfDeclaration | CompletionContext::InElements
 *                                expressionPath = {RangeFor<pr>}
 *                                propertyDeclaredType = RangeFor<int>
 *                                objectType = RangeFor<Element>
 * \endcode
 *
 * \code
 * component T < Element{ on complete : function(){ ... } }
 *                              ^ context = CompletionContext::InLeftOfDeclaration | CompletionContext::InElements
 *                                expressionPath = {RangeFor<com>}
 *                                propertyDeclaredType = RangeFor<on>
 *                                objectType = RangeFor<Element>
 * \endcode
 *
 * Note that for new component expressions, the objectType is 'T', not 'Element'
 *
 * \code
 * T{ prop : 200 }
 *      ^ context = CompletionContext::InLeftOfDeclaration | CompletionContext::InElements
 *        expressionPath = {RangeFor<pr>}
 *        objectType = RangeFor<T>
 * \endcode
 *
 * Imported types have a separate property that needs to be assigned, called objectImportNamespace:
 *
 * \code
 * import view as View
 * View.T{ prop : 200}
 *           ^ context = CompletionContext::InLeftOfDeclaration | CompletionContext::InElements
 *             expressionPath = {RangeFor<pr>}
 *             objectType = RangeFor<T>
 *             objectImportNamespace = RangeFor<View>
 * \endcode
 *
 *
 *
 *
 * Right of a declaration
 *
 * \code
 * T{ prop : this.font.size }
 *                      ^ context = CompletionContext::InRightOfDeclaration | CompletionContext::InElements
 *                        expressionPath = {RangeFor<this>, RangeFor<font>, RangeFor<s>}
 *                        objectType = RangeFor<T>
 *                        propertyPath = {RangeFor<prop>}
 * \endcode
 *
 *
 * \code
 * T{ int declaredProp : this.font.size }
 *                      ^ context = CompletionContext::InRightOfDeclaration | CompletionContext::InElements
 *                        expressionPath = {}
 *                        objectType = RangeFor<T>
 *                        propertyPath = {RangeFor<declaredProp>}
 *                        propertyDeclaredType = RangeFor<int>
 * \endcode
 *
 * \code // TODO
 * T{ fn declaredFunction(){ return 0 }
 *                              ^ context = CompletionContext::InRightOfDeclaration | CompletionContext::InElements
 *                                expressionPath = {RangeFor<ret>}
 *                                objectType = RangeFor<T>
 *                                propertyPath = {RangeFor<declaredFunction>}
 *                                propertyDeclaredType = RangeFor<fn>
 * \endcode
 *
 */

namespace lv{ namespace el{

std::set<std::string> CursorContext::keywords = {
    "as", "async", "await", "break", "case", "catch", "class",
    "const", "continue", "debugger", "default", "delete", "else",
    "extends", "finally", "for", "function", "get", "if", "import",
    "in", "instanceof", "let", "new", "of", "return", "set", "static",
    "switch", "target", "throw", "try", "typeof", "var", "void", "while",
    "with", "yield", "component", "fn", "constructor", "instance" };

CursorContext::CursorContext(int context,
        const std::vector<SourceRange> &expressionPath,
        const std::vector<SourceRange> &propertyPath,
        const SourceRange &propertyDeclaredType,
        const SourceRange &objectType,
        const SourceRange &objectImportNamespace)
    : m_context(context)
    , m_expressionPath(expressionPath)
    , m_propertyPath(propertyPath)
    , m_propertyDeclaredType(propertyDeclaredType)
    , m_objectType(objectType)
    , m_objectImportNamespace(objectImportNamespace)
{
}

CursorContext::~CursorContext(){
}

int CursorContext::context() const
{
    return m_context;
}

Utf8 CursorContext::contextString() const{
    std::string base;
    if ( m_context & CursorContext::InImport)
        base += "InImport";
    if ( m_context & CursorContext::InElements)
        base += base.empty() ? "InElements" : " | InElements";
    if ( m_context & CursorContext::InLeftOfDeclaration)
        base += base.empty() ? "InLeftOfDeclaration" : " | InLeftOfDeclaration";
    if ( m_context & CursorContext::InRightOfDeclaration)
        base += base.empty() ? "InRightOfDeclaration" : " | InRightOfDeclaration";
    if ( m_context & CursorContext::InStringLiteral)
        base += base.empty() ? "InString" : " | InString";
    return Utf8(base);
}

}} // namespace lv, el

==> live-elements-js-compiler-0.1.11.tgz: package/lib/lvelements/compiler/src/cursorcontext.cpp <==
/****************************************************************************
**
** Copyright (C) 2022 Dinu SV.
** This file is part of Livekeys Application.
**
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPLv3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl.html.
**
****************************************************************************/

#include "cursorcontext.h"

/**
 * \class lv::el::CursorContext
 * \brief Provides completion context info
 *
 * Cursor context provides information about a given cursor position within an elements document.
 * CursorContext is obtained from a parsed document, which extracts all the available information
 * from the AST.
 *
 * The class provides additional information depending on the context:
 *
 *
 *
 * Within an import:
 *
 * \code
 * import path.to.import
 *                  ^ context = CompletionContext::InImport | CompletionContext::InElements
 *                    expressionPath = {RangeFor<path>, RangeFor<to>, RangeFor<im>}
 * \endcode
 *
 * \code
 * import path.to.import
 *                ^ context = CompletionContext::InImport | CompletionContext::InElements
 *                  expressionPath = {RangeFor<path>, RangeFor<to>, Range()}
 * \endcode
 *
 * \code
 * import path.to.import
 *        ^ context = CompletionContext::InImport | CompletionContext::InElements
 *          expressionPath = {}
 * \endcode
 *
 * \code
 * import path.to.import
 *     ^ context = 0
 * \endcode
 *
 *
 *
 *
 * Within a string
 *
 * \code
 * var a = "example"
 *           ^ context = CompletionContext::InString
 * \endcode
 *
 *
 *
 *
 * Left of a declaration
 *
 * \code
 * component T < Element{ prop : 200 }
 *                          ^ context = CompletionContext::InLeftOfDeclaration | CompletionContext::InElements
 *                            expressionPath = {RangeFor<pr>}
 *                            objectType = RangeFor<Element>
 * \endcode
 *
 * \code
 * component T < Element{ int prop : 200 }
 *                              ^ context = CompletionContext::InLeftOfDeclaration | CompletionContext::InElements
 *                                expressionPath = {RangeFor<pr>}
 *                                propertyDeclaredType = RangeFor<int>
 *                                objectType = RangeFor<Element>
 * \endcode
 *
 * \code
 * component T < Element{ on complete : function(){ ... } }
 *                              ^ context = CompletionContext::InLeftOfDeclaration | CompletionContext::InElements
 *                                expressionPath = {RangeFor<com>}
 *                                propertyDeclaredType = RangeFor<on>
 *                                objectType = RangeFor<Element>
 * \endcode
 *
 * Note that for new component expressions, the objectType is 'T', not 'Element'
 *
 * \code
 * T{ prop : 200 }
 *      ^ context = CompletionContext::InLeftOfDeclaration | CompletionContext::InElements
 *        expressionPath = {RangeFor<pr>}
 *        objectType = RangeFor<T>
 * \endcode
 *
 * Imported types have a separate property that needs to be assigned, called objectImportNamespace:
 *
 * \code
 * import view as View
 * View.T{ prop : 200}
 *           ^ context = CompletionContext::InLeftOfDeclaration | CompletionContext::InElements
 *             expressionPath = {RangeFor<pr>}
 *             objectType = RangeFor<T>
 *             objectImportNamespace = RangeFor<View>
 * \endcode
 *
 *
 *
 *
 * Right of a declaration
 *
 * \code
 * T{ prop : this.font.size }
 *                      ^ context = CompletionContext::InRightOfDeclaration | CompletionContext::InElements
 *                        expressionPath = {RangeFor<this>, RangeFor<font>, RangeFor<s>}
 *                        objectType = RangeFor<T>
 *                        propertyPath = {RangeFor<prop>}
 * \endcode
 *
 *
 * \code
 * T{ int declaredProp : this.font.size }
 *                      ^ context = CompletionContext::InRightOfDeclaration | CompletionContext::InElements
 *                        expressionPath = {}
 *                        objectType = RangeFor<T>
 *                        propertyPath = {RangeFor<declaredProp>}
 *                        propertyDeclaredType = RangeFor<int>
 * \endcode
 *
 * \code // TODO
 * T{ fn declaredFunction(){ return 0 }
 *                              ^ context = CompletionContext::InRightOfDeclaration | CompletionContext::InElements
 *                                expressionPath = {RangeFor<ret>}
 *                                objectType = RangeFor<T>
 *                                propertyPath = {RangeFor<declaredFunction>}
 *                                propertyDeclaredType = RangeFor<fn>
 * \endcode
 *
 */

namespace lv{ namespace el{

std::set<std::string> CursorContext::keywords = {
    "as", "async", "await", "break", "case", "catch", "class",
    "const", "continue", "debugger", "default", "delete", "else",
    "extends", "finally", "for", "function", "get", "if", "import",
    "in", "instanceof", "let", "new", "of", "return", "set", "static",
    "switch", "target", "throw", "try", "typeof", "var", "void", "while",
    "with", "yield", "component", "fn", "constructor", "instance" };

CursorContext::CursorContext(int context,
        const std::vector<Utf8::Range> &expressionPath,
        const std::vector<Utf8::Range> &propertyPath,
        const Utf8::Range &propertyDeclaredType,
        const Utf8::Range &objectType,
        const Utf8::Range &objectImportNamespace)
    : m_context(context)
    , m_expressionPath(expressionPath)
    , m_propertyPath(propertyPath)
    , m_propertyDeclaredType(propertyDeclaredType)
    , m_objectType(objectType)
    , m_objectImportNamespace(objectImportNamespace)
{
}

CursorContext::~CursorContext(){
}

int CursorContext::context() const
{
    return m_context;
}

Utf8 CursorContext::contextString() const{
    std::string base;
    if ( m_context & CursorContext::InImport)
        base += "InImport";
    if ( m_context & CursorContext::InElements)
        base += base.empty() ? "InElements" : " | InElements";
    if ( m_context & CursorContext::InLeftOfDeclaration)
        base += base.empty() ? "InLeftOfDeclaration" : " | InLeftOfDeclaration";
    if ( m_context & CursorContext::InRightOfDeclaration)
        base += base.empty() ? "InRightOfDeclaration" : " | InRightOfDeclaration";
    if ( m_context & CursorContext::InStringLiteral)
        base += base.empty() ? "InString" : " | InString";
    return Utf8(base);
}

}} // namespace lv, el
